//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include "CityOccupationDialog.h"
#include <SDL_image.h>
#include "citylist.h"
#include "playerlist.h"
#include "File.h"

using namespace std;

#define debug(x) {cerr<<__FILE__<<": "<<__LINE__<<": "<<x<<endl;}
//#define debug(x)

CityOccupationDialog::CityOccupationDialog(City* city)
    :PG_Window(0, PG_Rect((PG_Application::GetScreenWidth()-320)/2,
                (PG_Application::GetScreenHeight()-340)/2, 320, 340),
                _("Defeated city"), WF_MODAL), d_b_pillage(0), d_city(city)
{
    d_background = File::getMiscPicture("city_occupied.jpg", false);
    SetBackground(d_background, 2);

    d_l_msg = new PG_Label(this, PG_Rect(10, 250, 280, 20), _("Choose your way!"));
    d_l_msg->SetAlignment(PG_TA_CENTER);

    //crude hack, for the realplayer we throw the dice here if he may pillage
    //the city
    bool maypillage = false;
    if (city->getArmytype(0) != -1)
    //city has at least one production
    {
        maypillage = true;
    }

    d_b_occupy = new PG_Button(this, 0, PG_Rect(20, my_height - 40, 80, 30), _("Occupy"));
    d_b_raze = new PG_Button(this, 0, PG_Rect(220, my_height - 40, 80, 30), _("Raze"));
    d_b_occupy->SetEventObject(MSG_BUTTONCLICK, this,
            (MSG_CALLBACK_OBJ)&CityOccupationDialog::b_occupyClicked);
    d_b_raze->SetEventObject(MSG_BUTTONCLICK, this,
            (MSG_CALLBACK_OBJ)&CityOccupationDialog::b_razeClicked);

    if (maypillage)
    {
        d_b_pillage = new PG_Button(this, 0, PG_Rect(120, my_height - 40, 80, 30), _("Pillage"));
        d_b_pillage->SetEventObject(MSG_BUTTONCLICK, this,
                (MSG_CALLBACK_OBJ)&CityOccupationDialog::b_pillageClicked);
    }
}

CityOccupationDialog::~CityOccupationDialog()
{
    delete d_b_occupy;
    delete d_b_pillage;
    delete d_b_raze;
    delete d_l_msg;
    SDL_FreeSurface(d_background);
}

PARAGUI_CALLBACK(CityOccupationDialog::b_occupyClicked)
{
    Playerlist::getInstance()->getActiveplayer()->cityOccupy(d_city);
    SendMessage(this, MSG_MODALQUIT, 0, 0);
    return true;
}

PARAGUI_CALLBACK(CityOccupationDialog::b_pillageClicked)
{
    Playerlist::getInstance()->getActiveplayer()->cityPillage(d_city);
    SendMessage(this, MSG_MODALQUIT, 0, 0);
    return true;
}

PARAGUI_CALLBACK(CityOccupationDialog::b_razeClicked)
{
    Playerlist::getInstance()->getActiveplayer()->cityRaze(d_city);
    SendMessage(this, MSG_MODALQUIT, 0, 0);
    return true;
}

// End of file
