//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include "FightDialog.h"
#include <pgapplication.h>
#include "stacklist.h"
#include "stack.h"
#include "army.h"
#include "player.h"
#include "GameMap.h"

using namespace std;

//#define debug(x) {cerr<<__FILE__<<": "<<__LINE__<<": "<<x<<endl;}
#define debug(x)

// The PG_Rect given as parameter for PG_ThemeWidget isn't important,
// because we override anyway with MoveWindow & SizeWindow
FightDialog::FightDialog (std::vector<Stack*>& attacker,
            std::vector<Stack*>& defender, PG_Widget* parent,
            bool def_bonus)
    :PG_ThemeWidget(parent, PG_Rect(), true), d_attacker(attacker),
    d_defender(defender), d_def_bonus(def_bonus), d_turn(0),
    d_result(Fight::NONE)
{
    initScreen();
}

FightDialog::FightDialog(Stack* attacker, Stack*defender, PG_Widget* parent,
            bool def_bonus)
    :PG_ThemeWidget(parent, PG_Rect(), true), d_attacker(1),
    d_defender(1), d_def_bonus(def_bonus), d_turn(0), d_result(Fight::NONE)
{
    d_attacker[0] = attacker;
    d_defender[0] = defender;

    initScreen();
}

FightDialog::~FightDialog()
{
    delete d_l_turn;
}

void FightDialog::initScreen()
{
    int width = 8*45 + 20;
    int height = (d_defender.size() + 1)*70 + 18;
    MoveWindow((PG_Application::GetScreenWidth() - width) / 2,
            (PG_Application::GetScreenHeight() - height) / 2);
    SizeWindow(width, height);

    d_l_turn = new PG_Label(this, PG_Rect(width / 2 - 5,
                height - 16, 40, 18), "0");
}

void FightDialog::battle()
{
    debug("battle()");

    PG_Point pos = d_defender[0]->getPos();
    Maptile* maptile = GameMap::getInstance()->getTile(pos.x,pos.y);

        // now create the class that does the real calculations
    Fight fight(maptile, d_attacker, d_defender, d_def_bonus);

    while ((d_result == Fight::NONE) && (fight.newRound()))
    {
        while ((!fight.isRoundFinished()) && (d_result == Fight::NONE))
        {
            d_result = fight.nextFight();
            Redraw();
        }
        // uncomment the delay here if it is too fast
        SDL_Delay(ROUND_DELAY);
        
        char buffer[10];
        sprintf(buffer, "%i", ++d_turn);
        d_l_turn->SetText(buffer);
    }
}

void FightDialog::eventDraw(SDL_Surface* surface, const PG_Rect& rect)
{
    if (d_result != Fight::NONE)
        //nothing useful to do then
        return;

    debug("eventDraw()");
    PG_ThemeWidget::eventDraw(surface, rect);
    DrawBorder(PG_Rect(0, rect.h - 18, rect.w, 18), 1);

    // draw the defenders
    int row = 0;
    for (std::vector<Stack*>::iterator it_stack = d_defender.begin();
            it_stack != d_defender.end(); ++it_stack, ++row)
    {
        int i = 0;
        for (Stack::iterator it = (*it_stack)->begin();
                it != (*it_stack)->end(); ++it)
        {
            if ((*it)->getHP() <= 0)
                //ignore stack when drawing
                continue;
                
            drawArmy(surface, i*45, row*70, *it);
            i++;
        }
        if (i == 0)
            //whole stack contains stiffs
            row--;
    }
    DrawBorder(PG_Rect(0, 0, my_width, row*70), 1);

    // draw the attackers
    int rowd = 0;
    for (std::vector<Stack*>::iterator it_stack = d_attacker.begin();
            it_stack != d_attacker.end(); ++it_stack, ++rowd)
    {
        int i = 0;
        for (Stack::iterator it = (*it_stack)->begin();
                it != (*it_stack)->end(); ++it)
        {
            if ((*it)->getHP() <= 0)
                continue;

            drawArmy(surface, i*45, row*70, *it);
            i++;
        }
        if (i == 0)
            rowd--;
    }
    DrawBorder(PG_Rect(0, row*70, my_width, rowd*70), 1);
}

void FightDialog::drawArmy(SDL_Surface* surface, int x, int y, Army* unit)
{
    SDL_Color red = {255, 0, 0};
    SDL_Color green = {0, 255, 0};

    SDL_BlitSurface(unit->getPixmap(), NULL, surface,
            PG_Rect(x + 10, y+ 6, 40, 40).SDLRect());

    // draw the hitpoints-bar
    float factor = (float)unit->getHP() / (float)unit->getMaxHp() * 38;
    DrawLine(x + 10, y + 54, x + 10 + 38, y + 54, red, 5);
    DrawLine(x + 10, y + 54, static_cast<unsigned int>(x + 10 + factor), y + 54, green, 5);

}

// End of file
