//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.*

#include "GamePreferencesDialog.h"
#include <pglabel.h>
#include <pgfilearchive.h>
#include "PG_FileDialog.h"
#include "GameScenario.h"
#include "MapConfDialog.h"
#include "player_preferences.h"
#include "File.h"
#include "Configuration.h"

using namespace std;

#define debug(x) {cerr<<__FILE__<<": "<<__LINE__<<": "<<x<<endl;}
//#define debug(x)

std::string turnmode_true = "";
std::string turnmode_false = "";

GamePreferencesDialog::GamePreferencesDialog(PG_Widget* parent, PG_Rect rect)
	:PG_ThemeWidget(parent, rect)
{
    debug("GamePreferencesDialog()");

    turnmode_true = _("player's turn");
    turnmode_false = _("new round");

    d_l_type = new PG_Label(this, PG_Rect(40, 10, 100, 20), _("Type"));
    d_l_name = new PG_Label(this, PG_Rect(145, 10, 100, 20), _("Name"));
    d_l_armyset = new PG_Label(this, PG_Rect(255, 10, 100, 20), _("Armyset"));

    // predefined names
    vector<string> nameList;
    nameList.push_back("Michael");
    nameList.push_back("Susi");
    nameList.push_back("Ulf");
    nameList.push_back("James");
    nameList.push_back("Vibhu");
    nameList.push_back("Tobias");
    nameList.push_back("Linus");
    nameList.push_back("Alan");
    for (int i = 0; i < 8; ++i)
    {
        PG_Rect p(10, 40 + i*35, 400, 25);
        Player_preferences::Type type = Player_preferences::ANY;
        if (i == 0) type = Player_preferences::HUMAN;
        if (i == 1) type = Player_preferences::HUMAN_OR_COMPUTER;
        d_player_preferences[i] = new Player_preferences(type, nameList[i], this, p);
        d_player_preferences[i]->playerDataChanged.connect(
                (SigC::slot(*this, &GamePreferencesDialog::slotPlayerDataChanged)));
	}

    d_b_random = new PG_Button(this, 0, PG_Rect(Width() - 310 , 10, 100, 25), _("Random"));
    d_b_random->SetToggle(true);
    d_b_load = new PG_Button(this, 1, PG_Rect(Width() - 210, 10, 100, 25), _("Load Map"));
    d_b_load->SetToggle(true);

    //choose tileset and turn mode
    d_l_tiles = new PG_Label(this, PG_Rect(35, 330, 120, 20), _("Choose Tileset"));
    d_d_tiles = new PG_DropDown(this, 3, PG_Rect(10, 360, 150, 30));
    File::scanTilesets(d_d_tiles); 

    //set the turn mode
    d_l_turnmode = new PG_Label(this, PG_Rect(175, 330, 290, 20),
                                _("Process armies at beginning of"));
    d_d_turnmode = new PG_DropDown(this, 4, PG_Rect(175, 360, 170, 30));
    d_d_turnmode->AddItem(turnmode_true.c_str());
    d_d_turnmode->AddItem(turnmode_false.c_str());
    d_d_turnmode->SetText(turnmode_true.c_str());

    
    d_edit = new PG_LineEdit(this, PG_Rect(Width() - 310, 60, 200, 25));
    d_edit->SetText("random.map");
    d_b_browse = new PG_Button(this, 2, PG_Rect(Width() - 100, 60, 90, 25), _("Browse"));
    d_grass = new TerrainConfig(this, PG_Rect(Width() - 310, 60, 300, 20),
                _("Grass"), 0, 99, 78);
    d_water = new TerrainConfig(this, PG_Rect(Width() - 310, 90, 300, 20),
                _("Water"), 0, 99, 7);
    d_swamp = new TerrainConfig(this, PG_Rect(Width() - 310, 120, 300, 20),
                _("Swamp"), 0, 99, 2);
    d_forest = new TerrainConfig(this, PG_Rect(Width() - 310, 150, 300, 20),
                _("Forest"), 0, 99, 3);
    d_hills = new TerrainConfig(this, PG_Rect(Width() - 310, 180, 300, 20),
                _("Hills"), 0, 99, 5);
    d_mountains = new TerrainConfig(this, PG_Rect(Width() - 310, 210, 300, 20),
                _("Mountains"), 0, 99, 5);
    d_cities = new TerrainConfig(this, PG_Rect(Width() - 310, 240, 300, 20),
                _("Cities"), 10, 30, 20);
    d_ruins = new TerrainConfig(this, PG_Rect(Width() - 310, 270, 300, 20),
                _("Ruins"), 15, 40, 25);
    d_temples = new TerrainConfig(this, PG_Rect(Width() - 310, 300, 300, 20),
                _("Temples"), 15, 40, 25);

    d_b_ok = new PG_Button(this, 3, PG_Rect(10, Height() - 40, Width() - 20, 30), _("Start Game"));

    //the buttons for the map size
    d_b_normalsize = new PG_Button(this, 5, PG_Rect(Width() - 250, 350, 150, 30), _("Normal size"));
    d_b_normalsize->SetToggle(true);
    d_b_smallsize = new PG_Button(this, 6, PG_Rect(Width() - 250, 390, 150, 30), _("Small size"));
    d_b_smallsize->SetToggle(true);
    d_b_tinysize = new PG_Button(this, 7, PG_Rect(Width() - 250, 430, 150, 30), _("Tiny size"));
    d_b_tinysize->SetToggle(true);
    d_b_normalsize->SetPressed(true);

    
    // if the previous map is still available use this it as default setting
    std::string randommap = File::getSavePath() + "random.map";
	if (PG_FileArchive::Exists(randommap.c_str()))
	{
        d_DialogType = GameScenario::LOADED;
	}
    else
    {
        d_DialogType = GameScenario::RANDOM;
    }

    d_b_random->SetEventObject(MSG_BUTTONCLICK, this,
            (MSG_CALLBACK_OBJ)&GamePreferencesDialog::randomClicked);
    d_b_load->SetEventObject(MSG_BUTTONCLICK, this,
            (MSG_CALLBACK_OBJ)&GamePreferencesDialog::loadClicked);
    d_b_browse->SetEventObject(MSG_BUTTONCLICK, this,
            (MSG_CALLBACK_OBJ)&GamePreferencesDialog::browseClicked);
    d_b_ok->SetEventObject(MSG_BUTTONCLICK, this,
            (MSG_CALLBACK_OBJ)&GamePreferencesDialog::okClicked);
    d_b_normalsize->SetEventObject(MSG_BUTTONCLICK, this,
            (MSG_CALLBACK_OBJ)&GamePreferencesDialog::sizeClicked);
    d_b_smallsize->SetEventObject(MSG_BUTTONCLICK, this,
            (MSG_CALLBACK_OBJ)&GamePreferencesDialog::sizeClicked);
    d_b_tinysize->SetEventObject(MSG_BUTTONCLICK, this,
            (MSG_CALLBACK_OBJ)&GamePreferencesDialog::sizeClicked);
}

GamePreferencesDialog::~GamePreferencesDialog()
{
    for (int i = 0; i < 8; i++)
        delete d_player_preferences[i];
    delete d_l_type;
    delete d_l_name;
    delete d_l_armyset;
    delete d_l_turnmode;
    delete d_d_turnmode;
    delete d_b_random;
    delete d_b_load;
    delete d_b_browse;
    delete d_b_ok;
    delete d_b_normalsize;
    delete d_b_smallsize;
    delete d_b_tinysize;
    delete d_grass;
    delete d_water;
    delete d_swamp;
    delete d_forest;
    delete d_hills;
    delete d_mountains;
    delete d_cities;
    delete d_ruins;
    delete d_temples;
}

void GamePreferencesDialog::initGUI()
{

    debug("initGUI");
    if (d_DialogType == GameScenario::LOADED)
    {
        d_grass->hide();
        d_water->hide();
        d_swamp->hide();
        d_forest->hide();
        d_hills->hide();
        d_mountains->hide();
        d_cities->hide();
        d_ruins->hide();
        d_temples->hide();
        d_b_normalsize->Hide();
        d_b_smallsize->Hide();
        d_b_tinysize->Hide();
        d_l_tiles->Hide();
        d_d_tiles->Hide();
        d_l_turnmode->Hide();
        d_d_turnmode->Hide();
        d_edit->Show();
        d_b_browse->Show();
        d_b_load->SetPressed(true);
        d_b_random->SetPressed(false);
    }
    else
    {
        d_grass->show();
        d_water->show();
        d_swamp->show();
        d_forest->show();
        d_hills->show();
        d_mountains->show();
        d_cities->show();
        d_ruins->show();
        d_temples->show();
        d_b_normalsize->Show();
        d_b_smallsize->Show();
        d_b_tinysize->Show();
        d_l_tiles->Show();
        d_d_tiles->Show();
        d_l_turnmode->Show();
        d_d_turnmode->Show();
        d_edit->Hide();
        d_b_browse->Hide();
        d_b_load->SetPressed(false);
        d_b_random->SetPressed(true);
    }
}

PARAGUI_CALLBACK(GamePreferencesDialog::browseClicked)
{
    debug("browse clicked")
    
    PG_FileDialog fd(0, PG_FileDialog::PG_OPEN, true, Configuration::s_savePath,
                        "*.map");
    
    fd.Show();
    fd.RunModal();
    fd.Hide();

    d_fileName = fd.getSelectedFile();
    if (!d_fileName.empty())
    {
        d_edit->SetText(d_fileName.c_str());
    }
    return true;
}

PARAGUI_CALLBACK(GamePreferencesDialog::randomClicked)
{
    d_DialogType = GameScenario::RANDOM;
    initGUI();
    return true;
}

PARAGUI_CALLBACK(GamePreferencesDialog::loadClicked)
{
    d_DialogType = GameScenario::LOADED;
    initGUI();
    return true;
}

PARAGUI_CALLBACK(GamePreferencesDialog::okClicked)
{
    //this is a workaround for a really strange bug where you would get a
    //segfault if you had activated an edit box and pressed the start button
    for (int i = 0; i < 8; i++)
        d_player_preferences[i]->Hide();
    d_edit->Hide();
        
    SendMessage(this, MSG_MODALQUIT, 0, 0);
    return true;
}

PARAGUI_CALLBACK(GamePreferencesDialog::sizeClicked)
{
    //set all buttons to not pressed except the caller
    d_b_normalsize->SetPressed(false);
    d_b_smallsize->SetPressed(false);
    d_b_tinysize->SetPressed(false);

    (static_cast<PG_Button*>(widget))->SetPressed(true);

    return true;
}

void GamePreferencesDialog::slotPlayerDataChanged()
{
    debug("slotPlayerDataChanged()");
    playerDataChanged.emit();
}

int GamePreferencesDialog::noPlayers()
{
	int n = 0;
	
	for (int i = 0; i < 8; i++)
		if (d_player_preferences[i]->isActive())
			n++;
	
	return n;
}

bool GamePreferencesDialog::fillData(CreateScenario* creator)
{
    //fill all player data
    SDL_Color color;
    Player::TYPE type;

    //first insert the neutral player
    color.r = color.g = color.b = 150;
    creator->addPlayer("Neutral", "default", color, Player::NEUTRAL);

    //then fill the other players
    for (int i = 0; i < 8; i++)
    {
        if (!d_player_preferences[i]->isActive())
            continue;

        switch(i)
        {
            case 0:color.r = color.g = 0; color.b = 255; break;
            case 1:color.r = 255; color.g = color.b = 0; break;
            case 2:color.r = 0; color.g = 255; color.b = 0; break;
            case 3:color.r = color.g = color.b = 190; break;
            case 4:color.r = 255; color.g = 0; color.b = 255;break;
            case 5:color.r = color.g = 255; color.b = 0; break;
            case 6:color.r = color.g = color.b = 255; break;
            case 7:color.r = color.g = color.b = 0; break;
        }
        
        if (d_player_preferences[i]->isComputer())
            type = Player::AI_FAST;
        else
            type = Player::HUMAN;

        creator->addPlayer(d_player_preferences[i]->get_name(),
                            d_player_preferences[i]->get_armyset(),
                            color, type);
    }

    //now fill in some map information
    //first, cities, temples, ruins
    creator->setMapTiles(std::string(d_d_tiles->GetText()));
    creator->setNoCities(d_cities->getValue());
    creator->setNoRuins(d_ruins->getValue());
    creator->setNoTemples(d_temples->getValue());

    //now the terrain. The scenario generator also accepts input with a sum of
    //more than 100%, so the thing is rather easy here
    creator->setPercentages(d_grass->getValue(), d_water->getValue(),
                            d_forest->getValue(), d_swamp->getValue(),
                            d_hills->getValue(), d_mountains->getValue());

    //finally, tell CreateScenario the map size
    if (d_b_normalsize->GetPressed())
        creator->setSize(100);
    if (d_b_smallsize->GetPressed())
        creator->setSize(70);
    if (d_b_tinysize->GetPressed())
        creator->setSize(50);

    //and tell it the turn mode
    creator->setTurnmode(false);
    if (std::string(d_d_turnmode->GetText()) == turnmode_true)
        creator->setTurnmode(true);
    
    return true;
}
// End of file
