//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include "QRuinSearch.h"
#include "signals.h"
#include "ruinlist.h"
#include "defs.h"
#include <iostream>
#include <sstream>

using namespace std;

//#define debug(x) {cerr<<__FILE__<<": "<<__LINE__<<": "<<x<<endl;}
#define debug(x)
#include <assert.h>
//=======================================================================
QuestRuinSearch::QuestRuinSearch(QuestsManager& mgr, 
                                 int hero) : Quest(mgr, hero)
{
    sruinSearched.connect( SigC::slot(*this, &QuestRuinSearch::ruinSearched));
    //    Ruin *ruin = NULL;

    Ruinlist *list = Ruinlist::getInstance();
    // how many unsearched ruins?
    unsigned int how_many = _howManyUnsearched();
    assert(how_many);

    unsigned int idx = rand() % how_many;

    debug("idx random = " << idx) ;
    unsigned int i = 0;

    Ruinlist::iterator it;
    for (it = list->begin(); it != list->end(); ++it)
    {
        if ((*it).isSearched() == 0)
        {
            if (i == idx)
                break;
            i++;
        }
    }
    d_ruin_id = (*it).getId();
    debug("ruin_id = " << d_ruin_id);
    _initDescription(d_description, (*it));
}
//=======================================================================
unsigned int QuestRuinSearch::_howManyUnsearched()
{
Ruinlist *list = Ruinlist::getInstance();
unsigned int how_many = 0;

    Ruinlist::iterator it;
    for (it = list->begin(); it != list->end(); ++it)
        if ((*it).isSearched() == 0)
            how_many++;
    debug("Unsearched ruins: " << how_many 
          << " (all ruins = " << list->size() << ")" << how_many);
    return how_many;
}
//=======================================================================
void QuestRuinSearch::_initDescription(std::string& d, Ruin& ruin)
{
    std::stringstream s;
    s << _("You must search the ruin (id = ") << ruin.getId() << ")";
    s << _(" located at (") << ruin.getPos().x << ", "
                         << ruin.getPos().y << ")";
    d = s.str();
}
//=======================================================================
QuestRuinSearch::QuestRuinSearch(QuestsManager& q_mgr, int heroId,
                                 XML_Helper* helper) 
                                 : Quest(q_mgr, heroId)
{
    sruinSearched.connect( SigC::slot(*this, &QuestRuinSearch::ruinSearched));
    // read ruin id from XML saved data:
    helper->getData(d_ruin_id, "ruin");
    // find a ruin object in the ruinlist:
    Ruin *ruin = NULL;
    Ruinlist *list = Ruinlist::getInstance();
    Ruinlist::iterator it;
    for (it = list->begin(); (it != list->end()) && (ruin == NULL); ++it)
        if ((int)(*it).getId() == d_ruin_id)
            ruin = &(*it);

    assert(ruin);
    // finish initialization:
    _initDescription(d_description, *ruin);
}
//=======================================================================
void QuestRuinSearch::getProgress(std::string& dst)
{
    dst += _("You aren't afraid of doing it, do you?");
}
//=======================================================================
bool QuestRuinSearch::isFeasible()
{
    return (_howManyUnsearched() > 0);
}
//=======================================================================
bool QuestRuinSearch::save(XML_Helper* helper)
{
    return helper->saveData("ruin", d_ruin_id);
}
//=======================================================================
void QuestRuinSearch::ruinSearched(int ruin_id, int hero_id)
{
    if (pending == 0)
        return;
    debug("ruin searched: " << ruin_id << " by hero " << hero_id);
    debug("our hero: " << getHeroId() << "(" << this << ")");
    if ((hero_id == getHeroId()) && (ruin_id == d_ruin_id))
    {
        debug("CONGRATULATIONS: QUEST 'RUIN SEARCH' IS COMPLETED!");
        questCompleted();                 
    }
}
//=======================================================================
void QuestRuinSearch::getSuccessMsg(std::queue<std::string>& msgs)
{
    msgs.push(_("The priests thank you for exploring this gloomy place."));
    msgs.push(_("Now they can set a temple here."));
}
