//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include "RAddUnit.h"
#include "playerlist.h"
#include "defs.h"
#include "stacklist.h"
#include "stack.h"

RAddUnit::RAddUnit(Army* army, unsigned int player, PG_Point pos)
    :Reaction(ADDUNIT), d_army(army), d_player(player), d_pos(pos)
{
}

RAddUnit::RAddUnit(XML_Helper* helper)
    :Reaction(ADDUNIT), d_army(0)
{
    int i;
    
    helper->getData(d_player, "player");
    helper->getData(i, "x");    d_pos.x = i;
    helper->getData(i, "y");    d_pos.y = i;

    //we use the army loading constructor to load the army; however, if we want
    //to use the save function of the army as well, we have to put the army
    //data in a separate tag and need to load it the complex way...
//helper->registerTag("army", SigC::slot((*this), &RAddUnit::loadArmy));
}

RAddUnit::~RAddUnit()
{
    if (d_army != 0)
        delete d_army;
}

bool RAddUnit::save(XML_Helper* helper)
{
    bool retval = true;

    retval &= helper->openTag("reaction");
    retval &= helper->saveData("type", std::string("RADDUNIT"));
    retval &= helper->saveData("player", d_player);
    retval &= helper->saveData("x", d_pos.x);
    retval &= helper->saveData("y", d_pos.y);

    retval &= d_army->save(helper);

    retval &= helper->closeTag();
    
    return retval;
}

bool RAddUnit::trigger()
{
    Player* p = getPlayer();
    PG_Point point;
    point.x = d_pos.x+1; point.y = d_pos.y;
    
    //We do the following:
    //First, let us create a stack at some random position. Then we look if the
    //position where we want to place the new stack is free. If so, we place the
    //stack there, else we do the same for all surrounding positions etc.
    
    //First, create a new stack NOT AT THE POSITION!
    Stack* s = new Stack(p, point);
    s->push_back(d_army);

    //Now we can use the stack's path and the getObjectAt() function of the
    //stacklist to determine where to place the stack. First, try it at the
    //original position.
    Stack* test = Stacklist::getObjectAt(d_pos);
    if (!test)
    {
    }
    
    return true;
}

Player* RAddUnit::getPlayer()
{
    Playerlist* plist = Playerlist::getInstance();
    for (Playerlist::iterator it = plist->begin(); it != plist->end(); it++)
        if ((*it)->getId() == d_player)
            return (*it);
    
    return 0;
}

bool RAddUnit::loadArmy(std::string tag, XML_Helper* helper)
{
    if (tag != "army")
    {
        std::cerr <<_("RAddUnit: got strange tag: ") <<tag <<std::endl;
        return false;
    }

   // d_army = new Army(helper);
    return true;
}
