//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef CITIES_REPORT_H
#define CITIES_REPORT_H

#include <sigc++/sigc++.h>
#include <pgwindow.h>
#include <pgeventobject.h>
#include <pgbutton.h>
#include <pglabel.h>
#include "citylist.h"
#include "playerlist.h"

/** Informational dialog about city status
  * 
  * This dialog is to be found under the menu point reports/cities. It displays
  * information about 4 cities at once and has an OK, an up and a down button.
  *
  * It contains subwidgets (CityItem instances) which care for the actual
  * display, the class itself just cares for filling the city items with the
  * correct city data.
  */

class CityItem;    //internal data item, further info in citiesreport.cpp,

class CitiesReport : public PG_EventObject, public PG_Window,
                     public SigC::Object
{
    public:
        //! paragui constructor
        CitiesReport(PG_Widget* parent, PG_Rect rect);
        ~CitiesReport();

        PARAGUI_CALLBACK(b_okClicked);
        PARAGUI_CALLBACK(b_upClicked);
        PARAGUI_CALLBACK(b_downClicked);

    private:
        //! Fills the data items beginning with the current index
        void fillCityItems();

        PG_Button* d_b_ok;
        PG_Button* d_b_up;
        PG_Button* d_b_down;
        PG_Label* d_l_number;
        CityItem* d_items[4];
        int d_index;
        std::vector<City*> d_citylist;
};

#endif /* CITIES_REPORT_H */
