//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.


//This file contains the various macros used within freelords.

#define FREELORDS_VERSION "0.3.3"
#define FREELORDS_SAVEGAME_VERSION "0.3.3"

#ifndef DEFINITIONS_H
#define DEFINITIONS_H

#define ENABLE_NLS 1 // Very important to activate localization
#include <libintl.h>
#define _(string) gettext(string) // Macro for the gettext

#include <paragui.h>
#include <list>

//ARMY_* these macros define special abilities of different armies. 

#define ARMY_NONE 0
#define ARMY_SHIP (1<<1)	//unit can only move on water
#define ARMY_HERO (1<<2)	//units has hero bonus (addit. strength for 
				//other units in combat)
#define ARMY_CAVALRY (1<<3)	//cavalry unit (+1 strength on grass)
#define ARMY_ANTICAVALRY (1<<4)	//double strength against cavalry
#define ARMY_HEALFAST (1<<5)	//+1 HP every 2nd fight turn and at end of fight
#define ARMY_EVADE (1<<6)	//escape first attack during battle round
#define ARMY_CRITICAL (1<<7)	//unit can do critical hits (instant death of
				//enemy regardless of hitpoints)
#define ARMY_HAS_HERO (1<<8)	//there's a hero in the stack (used during 
				//battle calculations UPDATE: unused
#define ARMY_IS_LOADED (1<<9)	//army is transported by another unit
#define ARMY_TRANSPORT_1 (1<<10)//army can hold one other unit
#define ARMY_TRANSPORT_2 (1<<11)//the same with two...
#define ARMY_TRANSPORT_3 (1<<12)//...and three units

#define ARMY_TRANSPORT ARMY_TRANSPORT_1&ARMY_TRANSPORT_2&ARMY_TRANSPORT_3

//-----------------------------------------------------------------------------
//These macros qualify terrain and armies. If an army and the maptile it moves 
//on have the same move bonus set, the unit needs two move points on that terain
//regardless of the number of MP's the terrain typically demands

#define MAPTILE_NONE 0
#define MAPTILE_GRASS MAPTILE_NONE
#define MAPTILE_WATER 1
#define MAPTILE_FOREST 2
#define MAPTILE_HILL 4
#define MAPTILE_SWAMP 8

#define CHARACTER_GRASS '.'
#define CHARACTER_HILLS 'h'
#define CHARACTER_WATER '~'
#define CHARACTER_FOREST '$'
#define CHARACTER_SWAMP '_'
#define CHARACTER_MOUNTAINS 'M'

#define CHARACTER_CITY_MAJOR 'C'
#define CHARACTER_CITY_MINOR 'c'
#define CHARACTER_TEMPLE 't'
#define CHARACTER_RUIN 'r'

//-----------------------------------------------------------------------------
//some typedefs for quests

typedef struct
{
	unsigned int type;
	unsigned int subtype;
	int amount;
} fl_Treasure;

#define TREASURE_GOLD 1

//-----------------------------------------------------------------------------
//typedefs and macros for the actionlist (i.e. player's lists which hold 
//information about the player's actions)

const unsigned int ACTION_STACK_MOVED_ONE_STEP=1;
const unsigned int ACTION_STACK_SPLIT=2;
const unsigned int ACTION_STACK_SEARCH_RUIN=3;
const unsigned int ACTION_STACK_FIGHT=4;
const unsigned int ACTION_STACK_FIGHT_CITY=5;
const unsigned int ACTION_STACK_JOIN=6;
const unsigned int ACTION_STACK_VISIT_TEMPLE=7;
const unsigned int ACTION_CITY_OCCUPY=8;
const unsigned int ACTION_CITY_RAZE=9;
const unsigned int ACTION_CITY_UPGRADE_DEFENSE=10;
const unsigned int ACTION_CITY_BUY_PRODUCTION=11;
const unsigned int ACTION_CITY_CHANGE_PRODUCTION=12;
const unsigned int ACTION_REWARD=13;
const unsigned int ACTION_CITY_PILLAGE=14;

//-----------------------------------------------------------------------------
//some standard timers. They can easier be changed here than somewhere deep
//within the code, and sometimes you have to tweak them a little bit.
const unsigned int TIMER_BIGMAP_SELECTOR=500;
const unsigned int TIMER_BIGMAP_SCROLLING=30000;   //microseconds
const unsigned int TIMER_SMALLMAP_REFRESH=5;

#endif // DEFINITIONS_H
