//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include "splash.h"
#include <iostream>
#include <SDL_image.h>
#include "MainWindow.h"
#include "defs.h"
#include "armysetlist.h"
#include "stacklist.h"
#include "playerlist.h"
#include "Configuration.h"
#include "MultiPlayerModeDialog.h"
#include "PG_FileDialog.h"
#include "CreateScenario.h"
#include "File.h"
#include "REndGame.h"

using namespace std;

#define debug(x) {cerr<<__FILE__<<": "<<__LINE__<<": "<<x<<endl;}
//#define debug(x)

Splash::Splash(PG_Application* app, const PG_Rect& rect)
    :PG_ThemeWidget(0, rect), d_mainWindow(0)
{
    d_app = app;
    d_background = File::getMiscPicture("splash_screen.jpg", false);
    SetBackground(d_background, 2);
    
    d_b_campaign = new PG_Button(this, 0,
            PG_Rect(rect.w - 210, rect.h - 215, 180, 30), _("New Campaign"));
    d_b_singleplayer_game = new PG_Button(this, 1,
            PG_Rect(rect.w - 210, rect.h - 180, 180, 30), _("New Game"));
    d_b_load_game = new PG_Button(this, 4,
            PG_Rect(rect.w - 210, rect.h - 145, 180, 30), _("Load Game"));
    d_b_tutorial = new PG_Button(this, 3,
            PG_Rect(rect.w - 210, rect.h - 110, 180, 30), _("Tutorial"));
    d_b_multiplayer_game = new PG_Button(this, 2,
            PG_Rect(rect.w - 210, rect.h - 75, 180, 30), _("New Multiplayer Game"));
    d_b_quit = new PG_Button(this, 5,
            PG_Rect(rect.w - 210, rect.h - 40, 180, 30), _("Quit"));

    d_b_campaign->SetEventObject(MSG_BUTTONCLICK, this,
            (MSG_CALLBACK_OBJ)&Splash::b_campaignClicked);
    d_b_singleplayer_game->SetEventObject(MSG_BUTTONCLICK, this,
            (MSG_CALLBACK_OBJ)&Splash::b_singleplayerGameClicked);
    d_b_multiplayer_game->SetEventObject(MSG_BUTTONCLICK, this,
            (MSG_CALLBACK_OBJ)&Splash::b_multiplayerGameClicked);
    d_b_load_game->SetEventObject(MSG_BUTTONCLICK, this,
            (MSG_CALLBACK_OBJ)&Splash::b_loadGameClicked);
    d_b_tutorial->SetEventObject(MSG_BUTTONCLICK, this,
            (MSG_CALLBACK_OBJ)&Splash::b_tutorialClicked);
    d_b_quit->SetEventObject(MSG_BUTTONCLICK, this,
            (MSG_CALLBACK_OBJ)&Splash::b_quitClicked);
}

Splash::~Splash()
{
    SDL_FreeSurface(d_background);
    clearData();
    delete d_b_campaign;
    delete d_b_singleplayer_game;
    delete d_b_multiplayer_game;
    delete d_b_load_game;
    delete d_b_tutorial;
    delete d_b_quit;
}

PARAGUI_CALLBACK(Splash::b_quitClicked)
{
    d_app->Quit();
    return true;
}

PARAGUI_CALLBACK(Splash::b_singleplayerGameClicked)
{
    clearData();
    newGame(GameScenario::SINGLE_PLAYER);
    return true;
}

void Splash::newGame(GameScenario::GameType gameType, std::string ip, int port)
{
    if (gameType == GameScenario::SINGLE_PLAYER)
    {
        CreateScenario* creator = new CreateScenario(this);
        GamePreferencesDialog* dialog = 0;
        
        //CreateScenario requires to define a maptype
        creator->setMaptype(CreateScenario::NORMAL);

        if (Configuration::s_testing)
        {
            //we want to set up a testing game with two AI players
            SDL_Color color;
            
            //first player:
            color.r = color.g = 0;      color.b = 255;
            creator->addPlayer("Michael", "default", color, Player::AI_FAST);

            //second player
            color.r = 255;              color.g = color.g = 0;
            creator->addPlayer("Susi", "default", color, Player::AI_FAST);

            //neutral player...
            color.r = color.g = color.b = 150;
            creator->addPlayer("Neutral", "default", color, Player::NEUTRAL);
        }
        else
        {
            //the player has to choose which player to activate etc.
        
            dialog = new GamePreferencesDialog(0, PG_Rect(0, 0,
                    PG_Application::GetScreenWidth(),
                    PG_Application::GetScreenHeight()));

            dialog->Show();
            dialog->initGUI();
            dialog->RunModal();
            dialog->Hide();

            dialog->fillData(creator);

        }

        //now create the map and dump the created map under ~savepath/random.map
        std::string randomfilename = File::getSavePath();
        randomfilename += "random.map";
        
        //!dialog means testing game
        if (!dialog || (dialog->getMapType() != GameScenario::LOADED))
        {
            creator->create();
            creator->dump(randomfilename);
        }

        delete creator;
        if (dialog)
            delete dialog;

        //and finally load the "savegame", set up the main window and start
        bool broken;
        GameScenario* gameScenario = new GameScenario(randomfilename, broken);

        if (broken)
        {
            std::cerr <<_("random.map was broken when re-reading. Exiting...\n");
            exit(-1);
        }

        //set up misc stuff
        REndGame::connect(SigC::slot((*this), &Splash::gameFinished));

        Hide();
        
        d_mainWindow = new MainWindow(gameScenario,
                                        PG_Rect(0, 0, Width(), Height()));
        d_mainWindow->squitting.connect(SigC::slot((*this),
                                        &Splash::gameFinished));
        d_mainWindow->Show();
        d_mainWindow->startGame();

        return;
    }
}

PARAGUI_CALLBACK(Splash::b_multiplayerGameClicked)
{
    clearData();

    MultiPlayerModeDialog dialog(this, PG_Rect(20, 20, 350, 230));
    dialog.Show();
    dialog.RunModal();
    dialog.Hide();

    if (dialog.getResult())
    {
        if (dialog.getMode())
        {
            newGame(GameScenario::MULTI_PLAYER_HOST, "", dialog.getPort());
        }
        else
        {  
            newGame(GameScenario::MULTI_PLAYER_CLIENT, dialog.getIP(), dialog.getPort());
        }
    }

    return true;
}

PARAGUI_CALLBACK(Splash::b_loadGameClicked)
{
    clearData();

    PG_FileDialog opendialog(0, PG_FileDialog::PG_OPEN, true,
            Configuration::s_savePath, "*.sav", _("FreeLords savegames"));
    opendialog.Show();
    opendialog.RunModal();
    std::string savegame = opendialog.getSelectedFile();

    //no file selected
    if (savegame.empty())
    {
        return true;
    }
    
    bool broken;
    GameScenario* gameScenario = new GameScenario(savegame, broken);

    if (broken)
    {
        std::cerr <<_("Couldn't load savegame ") <<savegame <<_(". Exiting.\n");
        exit(-1);
    }

    REndGame::connect(SigC::slot((*this), &Splash::gameFinished));

    Hide();
        
    d_mainWindow = new MainWindow(gameScenario, PG_Rect(0, 0, Width(), Height()));
    d_mainWindow->squitting.connect(SigC::slot((*this), &Splash::gameFinished));
    d_mainWindow->Show();
    d_mainWindow->loadGame(savegame);

    return true;
}

PARAGUI_CALLBACK(Splash::b_tutorialClicked)
{
    debug(_("TUTORIAL Clicked!"));

    clearData();

    std::string savegame = Configuration::s_dataPath+string("/map/tutorial.map");

    bool broken;
    GameScenario* gameScenario = new GameScenario(savegame, broken);

    if (broken)
    {
        std::cerr <<_("Couldn't load savegame ") <<savegame <<_(". Exiting.\n");
        exit(-1);
    }

    REndGame::connect(SigC::slot((*this), &Splash::gameFinished));

    Hide();
        
    d_mainWindow = new MainWindow(gameScenario, PG_Rect(0, 0, Width(), Height()));
    d_mainWindow->squitting.connect(SigC::slot((*this), &Splash::gameFinished));
    d_mainWindow->Show();
    d_mainWindow->loadGame(savegame);
    d_mainWindow->startGame();
    return true;
}


PARAGUI_CALLBACK(Splash::b_campaignClicked)
{
    clearData();
    std::cerr << _("not implemented yet\n");
    return true;
}

void Splash::clearData()
{
    if (d_mainWindow)
    {
        delete d_mainWindow;
        d_mainWindow = 0;
    }
}

void Splash::gameFinished()
{
    debug("gameFinished()");

    //the memory for gameScenario and mainWindow is freed as soon as a
    //new game is started

    d_mainWindow->stopGame();
    d_mainWindow->Hide();

    Show();
}
